package wanijo.wanijo2.domain

import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertNotNull
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.test.annotation.DirtiesContext
import kotlin.test.assertEquals

@SpringBootTest
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_EACH_TEST_METHOD)
class DocumentDaoTest {

    @Autowired
    lateinit var adapter: DocumentDao

    @Test
    fun insertWorks() {
        assertEquals(0, adapter.findAll().count())

        adapter.save(
            Document(name = "name")
        )

        assertEquals(1, adapter.findAll().count())
    }

    @Test
    fun findByIdWorks() {
        val doc = Document(name = "name")

        adapter.save(doc)

        val savedDoc = adapter.findById(1)
        assertNotNull(savedDoc)
    }

    @Test
    fun idsAreGeneratedSequentially() {
        adapter.save(
            Document(name = "name1")
        )

        adapter.save(
            Document(name = "name2")
        )

        assertNotNull(adapter.findById(1))
        assertNotNull(adapter.findById(2))
    }

    @Test
    fun findByNameWorks() {
        adapter.save(
            Document(name = "findmich")
        )

        assertNotNull(adapter.findByName("findmich"))
    }

    @Test
    fun findsDocumentsWithTag() {
        adapter.save(
            Document(
                name = "Dok1",
                tags = setOf(Tag(name = "tAg1"))
            )
        )

        adapter.save(
            Document(
                name = "Dok2",
                tags = setOf(Tag(name = "taG2"))
            )
        )

        val docs = adapter.findByTagName("tag1")
        assertEquals(1, docs.size)
        assertEquals("Dok1", docs[0].name)
    }

    @Test
    fun tagsAreSaved() {
        adapter.save(
            Document(
                name = "dok1",
                tags = setOf(Tag(name = "tag1"), Tag(name = "tag2"))
            )
        )

        val doc = adapter.findAll()[0]
        assertEquals(2, doc.tags.size)
        assertEquals("tag1", doc.tags.first().name)
        assertEquals("tag2", doc.tags.last().name)
    }

}
